'use strict';

var vAPI = Object.create(null);

if ( document instanceof window.HTMLDocument ) {
	vAPI.browser = document.documentElement || document.createElement('div');

	if ( !window.requestAnimationFrame ) {
		window.requestAnimationFrame = function(cb) {
			return window.setTimeout(cb, 25);
		};
	}

	if ( vAPI.browser.style ) {
		vAPI._ = vAPI.browser.style;
		vAPI.browser = {
			wheel: 'onwheel' in vAPI.browser ? 'wheel' : 'mousewheel',
			transform: ('webkitTransform' in vAPI._ ? 'webkitT' : 't') + 'ransform',
			'transition-css': ('webkitTransition' in vAPI._
				? '-webkit-'
				: ''
			) + 'transition',
			transitionend: 'webkitTransition' in vAPI._
				? 'webkitTransitionEnd'
				: 'transitionend',
			'box-sizing-css': ('MozBoxSizing' in vAPI._
				? '-moz-'
				: 'WebkitBoxSizing' in vAPI._ ? '-webkit-' : ''
			) + 'box-sizing',
			'zoom-in': this.chrome || this.mx || this.safari
				? '-webkit-zoom-in'
				: 'zoom-in'
		};
		delete vAPI._;
	}
}

vAPI.buildNodes = function(host, nodes) {
	if ( !host || !Array.isArray(nodes) ) {
		return null;
	}

	if ( !nodes.length ) {
		return host;
	}

	var doc = host.ownerDocument;
	var fragment = doc.createDocumentFragment();

	for ( var i = 0, l = nodes.length; i < l; ++i ) {
		if ( !nodes[i] ) {
			continue;
		}

		if ( typeof nodes[i] === 'string' ) {
			fragment.appendChild(doc.createTextNode(nodes[i]));
			continue;
		}

		var node = doc.createElement(nodes[i].tag);

		if ( nodes[i].attrs ) {
			for ( var attr in nodes[i].attrs ) {
				// bypass CSP
				if ( attr === 'style' ) {
					node.style.cssText = nodes[i].attrs[attr];
				} else {
					node.setAttribute(attr, nodes[i].attrs[attr]);
				}
			}
		}

		if ( nodes[i].nodes ) {
			this.buildNodes(node, nodes[i].nodes);
		} else if ( nodes[i].text ) {
			node.textContent = nodes[i].text;
		}

		fragment.appendChild(node);
	}

	if ( fragment.childNodes.length ) {
		host.appendChild(fragment);
	}

	return host;
};

vAPI.firefox = true;

Object.defineProperty(vAPI, 'fullScreenElement', {
	get: function() {
		return document.mozFullScreenElement || null;
	}
});

Object.defineProperty(vAPI, 'mediaType', {get: function() {
	if ( this._mediaType !== void 0 ) {
		return this._mediaType;
	}

	var selector =
		'meta[content="width=device-width; height=device-height;"],' +
		'link[rel=stylesheet][href^="resource://gre/res/TopLevel"],' +
		'link[rel=stylesheet][href^="chrome://global/skin/media/TopLevel"]';

	var head = document.head;
	this._mediaType = '';

	if ( !head || head.querySelectorAll(selector).length !== 3 ) {
		return this._mediaType;
	}

	var media = document.querySelector(
		'body > img:first-child, ' +
		'body > video[controls][autoplay]:not([src]):empty'
	);

	if ( !media ) {
		return this._mediaType;
	}

	if ( media.src && media.src === window.location.href ) {
		return this._mediaType = 'img';
	}

	// When media is redirected the currentSrc doesn't change
	/*if ( media.parentNode.currentSrc !== location.href ) {
		return this._mediaType;
	}*/

	return this._mediaType = 'video';
}, set: function(type) {
	this._mediaType = type;
}});

vAPI.messaging = {
	listen: function(listener, once) {
		if ( this.listener ) {
			removeMessageListener();
		}

		if ( typeof listener !== 'function' ) {
			this.listener = null;
			return;
		}

		this.listener = function(e) {
			listener(JSON.parse(e));

			if ( once ) {
				vAPI.messaging.listen(null);
			}
		};

		addMessageListener(this.listener);
	},

	send: function(message, callback) {
		if ( typeof callback === 'function' ) {
			this.listen(callback, true);
		}

		sendAsyncMessage(_hostName_ + ':background', {
			listenerId: _sandboxId_,
			data: message,
			origin: window.location.href
		});
	},

	toggleListener: function(e) {
		if ( !vAPI.messaging.listener ) {
			return;
		}

		if ( e.type === 'pagehide' ) {
			removeMessageListener();
			return;
		}

		if ( e.persisted ) {
			addMessageListener(vAPI.messaging.listener);
		}
	}
};

window.addEventListener('pagehide', vAPI.messaging.toggleListener, true);
window.addEventListener('pageshow', vAPI.messaging.toggleListener, true);

if ( location.protocol === 'chrome:' && location.hostname === _hostName_ ) {
	vAPI.i18n = (function() {
		var stringBundle = Components
			.classes['@mozilla.org/intl/stringbundle;1']
			.getService(Components.interfaces.nsIStringBundleService)
			.createBundle(
				'chrome://' + location.host + '/locale/options.properties'
			);

		return function(s) {
			try {
				return stringBundle.GetStringFromName(s); // eslint-disable-line
			} catch ( ex ) {
				return s;
			}
		};
	})();

	vAPI.insertHTML = (function() {
		var io = Components.classes['@mozilla.org/network/io-service;1']
			.getService(Components.interfaces.nsIIOService);
		var parser = Components.classes['@mozilla.org/parserutils;1']
			.getService(Components.interfaces.nsIParserUtils);

		return function(node, html) {
			while ( node.firstChild ) {
				node.removeChild(node.firstChild);
			}

			node.appendChild(parser.parseFragment(
				html,
				parser.SanitizerAllowStyle,
				false,
				io.newURI(document.baseURI, null, null),
				document.documentElement
			));
		};
	})();
}
